@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/chts.c,v 1.27 2004/03/07 01:56:18 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/chts.c,v 1.28 2004/06/30 15:38:17 hazelsct Exp $
 
   This is the Cahn Hilliard timestepping code.  It is provided here as an
   example usage of the Illuminator Distributed Visualization Library.
@@ -215,7 +215,7 @@ int ch_ts_monitor (TS thets, int stepno, PetscScalar time, Vec X, void *ptr)
   if (user->save_data)
     {
       PetscReal deltat;
-      field_plot_type chtype=FIELD_SCALAR_CONTOURS;
+      field_plot_type chtype=FIELD_SCALAR;
       char filename [100], *paramdata [4],
 	*paramnames [4] = { "time", "timestep", "gamma", "kappa" };
 
