@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/illuminator.h,v 1.27 2004/11/04 22:47:45 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/illuminator.h,v 1.28 2005/04/19 01:35:14 hazelsct Exp $
 
   This is the interface for the Illuminator library.
 ***************************************/
@@ -21,6 +21,9 @@ typedef enum {
   /*+Ternary composition field with two components (third component is inferred
     from first two).+*/
   FIELD_TERNARY          = 0x10,
+  /*+Ternary composition with pseudo-components mapping onto a rectangle
+    instead of a triangle.+*/
+  FIELD_TERNARY_SQUARE   = 0x18,
   /*+Vector field.+*/
   FIELD_VECTOR           = 0x20,
   /*+Full ds*ds tensor field, e.g. transformation.+*/
