@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/illumulti.c,v 1.27 2004/06/30 15:38:18 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/illumulti.c,v 1.29 2005/04/19 00:49:29 hazelsct Exp $
 
   This file contains the functions
   +latex+{\tt IlluMultiSave()}, {\tt IlluMultiLoad()} and {\tt IlluMultiRead()}
@@ -48,6 +48,7 @@
 #include <libxml/tree.h>     /* To build the XML tree to store */
 #include <libxml/parser.h>   /* XML parsing */
 #include <stdio.h>           /* FILE, etc. */
+#include <errno.h>           /* errno */
 #include <string.h>          /* strncpy(), etc. */
 #include <stdlib.h>          /* malloc() and free() */
 
@@ -594,6 +595,7 @@ static int IlluMultiParseData
   int i;
   char filename [1000];
   FILE *infile;
+  size_t readoutput;
 
   if (compressed & COMPRESS_GZIP_MASK)
     {
@@ -601,14 +603,26 @@ static int IlluMultiParseData
 		    basename, rank) > 999)
 	return 1;
       if (!(infile = popen (filename, "r")))
-	return 1;
+	{
+	  char openerror [1050];
+	  snprintf (openerror, 1049,
+		    "CPU %d: error %d (%s) opening input pipe \"%s\"",
+		    rank, errno, strerror (errno), filename);
+	  SETERRQ (PETSC_ERR_FILE_OPEN, openerror);
+	}
     }
   else
     {
       if (snprintf (filename, 999, "%s.cpu%.4d.data", basename, rank) > 999)
 	return 1;
       if (!(infile = fopen (filename, "r")))
-	return 1;
+	{
+	  char openerror [1050];
+	  snprintf (openerror, 1049,
+		    "CPU %d: error %d (%s) opening input file %s",
+		    rank, errno, strerror (errno), filename);
+	  SETERRQ (PETSC_ERR_FILE_OPEN, openerror);
+	}
     }
 
   /* Read and store the data */
@@ -624,7 +638,7 @@ static int IlluMultiParseData
 	if (!(storage = (guint32 *) malloc
 	      (gridpoints * dof * sizeof (guint32))))
 	  return 1;
-	fread (storage, sizeof (guint32), gridpoints * dof, infile);
+	readoutput = fread (storage, sizeof (guint32), gridpoints*dof, infile);
 
 	if (wrongendian)
 	  {
@@ -643,7 +657,7 @@ static int IlluMultiParseData
 	if (!(storage = (guint16 *) malloc
 	      (gridpoints * dof * sizeof (guint16))))
 	  return 1;
-	fread (storage, sizeof (guint16), gridpoints * dof, infile);
+	readoutput = fread (storage, sizeof (guint16), gridpoints*dof, infile);
 
 	if (wrongendian)
 	  {
@@ -662,7 +676,7 @@ static int IlluMultiParseData
 	if (!(storage = (guint8 *) malloc
 	      (gridpoints * dof * sizeof (guint8))))
 	  return 1;
-	fread (storage, sizeof (guint8), gridpoints * dof, infile);
+	readoutput = fread (storage, sizeof (guint8), gridpoints*dof, infile);
 
 	for (i=0; i<gridpoints*dof; i++)
 	  globalarray[i] = fieldmin [i%dof] +
@@ -673,7 +687,8 @@ static int IlluMultiParseData
     default:
       {
 	/* TODO: check for different size data, complex */
-	fread (globalarray, sizeof (PetscScalar), gridpoints * dof, infile);
+	readoutput = fread (globalarray, sizeof (PetscScalar), gridpoints*dof,
+			    infile);
 
 	if (wrongendian)
 	  {
@@ -698,6 +713,15 @@ static int IlluMultiParseData
   else
     fclose (infile);
 
+  if (readoutput < gridpoints*dof)
+    {
+      char readerror [1050];
+      int readerr = ferror (infile);
+      snprintf (readerror,1049, "CPU %d: error %d (%s) during read, got %d of %d items",
+		rank, readerr, strerror (readerr), readoutput, gridpoints*dof);
+      SETERRQ (PETSC_ERR_FILE_READ, readerror);
+    }
+
   return 0;
 }
 
@@ -1002,7 +1026,12 @@ static int IlluMultiStoreXML
   if (snprintf (filename, 999, "%s.cpu%.4d.meta", basename, rank) > 999)
     return 1;
   if (!(outfile = fopen (filename, "w")))
-    return 1;
+    {
+      char openerror [1050];
+      snprintf (openerror,1049, "CPU %d: error %d (%s) opening output file %s",
+		rank, errno, strerror (errno), filename);
+      SETERRQ (PETSC_ERR_FILE_OPEN, openerror);
+    }
   xmlDocDump (outfile, thedoc);
   xmlFreeDoc (thedoc);
   fclose(outfile);
@@ -1050,6 +1079,7 @@ static int IlluMultiStoreData
   int i;
   char filename [1000];
   FILE *outfile;
+  size_t writeout;
 
   if (compressed & COMPRESS_GZIP_MASK)
     {
@@ -1067,14 +1097,26 @@ static int IlluMultiStoreData
 	    return 1;
 	}
       if (!(outfile = popen (filename, "w")))
-	return 1;
+	{
+	  char openerror [1050];
+	  snprintf (openerror, 1049,
+		    "CPU %d: error %d (%s) opening output pipe \"%s\"",
+		    rank, errno, strerror (errno), filename);
+	  SETERRQ (PETSC_ERR_FILE_OPEN, openerror);
+	}
     }
   else
     {
       if (snprintf (filename, 999, "%s.cpu%.4d.data", basename, rank) > 999)
 	return 1;
       if (!(outfile = fopen (filename, "w")))
-	return 1;
+	{
+	  char openerror [1050];
+	  snprintf (openerror, 1049,
+		    "CPU %d: error %d (%s) opening output file %s",
+		    rank, errno, strerror (errno), filename);
+	  SETERRQ (PETSC_ERR_FILE_OPEN, openerror);
+	}
     }
 
   switch (compressed & COMPRESS_INT_MASK)
@@ -1089,7 +1131,7 @@ static int IlluMultiStoreData
 	  storage [i] = (guint32)
 	    (4294967295. * (globalarray [i] - fieldmin [i%dof]) /
 	     (fieldmax [i%dof] - fieldmin [i%dof]));
-	fwrite (storage, sizeof (guint32),  gridpoints * dof, outfile);
+	writeout = fwrite (storage, sizeof (guint32), gridpoints*dof, outfile);
 	free (storage);
 	break;
       }
@@ -1103,7 +1145,7 @@ static int IlluMultiStoreData
 	  storage [i] = (guint16)
 	    (65535. * (globalarray [i] - fieldmin [i%dof]) /
 	     (fieldmax [i%dof] - fieldmin [i%dof]));
-	fwrite (storage, sizeof (guint16),  gridpoints * dof, outfile);
+	writeout = fwrite (storage, sizeof (guint16),  gridpoints*dof,outfile);
 	free (storage);
 	break;
       }
@@ -1117,18 +1159,30 @@ static int IlluMultiStoreData
 	  storage [i] = (guint8)
 	    (255. * (globalarray [i] - fieldmin [i%dof]) /
 	     (fieldmax [i%dof] - fieldmin [i%dof]));
-	fwrite (storage, sizeof (guint8),  gridpoints * dof, outfile);
+	writeout = fwrite (storage, sizeof (guint8),  gridpoints*dof, outfile);
 	free (storage);
 	break;
       }
     default:
-      fwrite (globalarray, sizeof (PetscScalar), gridpoints * dof, outfile);
+      writeout = fwrite (globalarray, sizeof (PetscScalar), gridpoints*dof,
+			 outfile);
     }
 
   if (compressed & COMPRESS_GZIP_MASK)
     pclose (outfile);
   else
     fclose (outfile);
+
+  if (writeout < gridpoints*dof)
+    {
+      char writerror [1050];
+      int writerr = ferror (outfile);
+      snprintf (writerror,1049, "CPU %d: error %d (%s) during write, sent %d of %d items",
+		rank, writerr, strerror (writerr), writeout, gridpoints*dof);
+      SETERRQ (PETSC_ERR_FILE_READ, writerror);
+    }
+
+  return 0;
 }
 
 
