@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/render.c,v 1.15 2004/11/30 14:35:54 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/render.c,v 1.17 2005/04/28 09:14:36 hazelsct Exp $
 
   This file contains the rendering code for Illuminator, which renders 2-D or
   3-D data into an RGB(A) unsigned char array (using perspective in 3-D).
@@ -113,6 +113,39 @@ static inline void pseudoternarycolor
 }
 
 
+#undef __FUNCT__
+#define __FUNCT__ "pseudoternarysquarecolor"
+
+/*++++++++++++++++++++++++++++++++++++++
+  This little function converts two composition values into an rgb color, with
+  blue, red, green and yellow indicating the corners.
+
+  PetscScalar A First ternary composition.
+
+  PetscScalar B Second ternary composition.
+
+  PetscScalar *scale Array: minimum and maximum first and second compositions
+  for scaling.
+
+  guchar *pixel  Address in rgb buffer where this pixel should be painted.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static inline void pseudoternarysquarecolor
+(PetscScalar A, PetscScalar B, PetscScalar *scale, guchar *pixel)
+{
+  PetscScalar x1, x2, inverse_det;
+
+  /* Transform A,B into x1,x2 based on corners */
+  x1 = (A-scale[0])/(scale[1]-scale[0]);
+  x2 = (B-scale[2])/(scale[3]-scale[2]);
+
+  /* Now colorize */
+  *pixel++ = 255*x2;
+  *pixel++ = 255*x1;
+  *pixel++ = 255*(1.-x1)*(1.-x2);
+}
+
+
 #undef __FUNCT__
 #define __FUNCT__ "pseudohueintcolor"
 
@@ -264,6 +297,16 @@ int render_scale_2d (guchar *rgb, int rwidth, int rheight, int bytes_per_pixel,
 	     bytes_per_pixel);
       return 0;
 
+    case FIELD_TERNARY_SQUARE:
+      scale[0] = scale[2] = 0.;
+      scale[1] = scale[3] = 1.;
+      for (j=0; j<rheight; j++)
+	for (i=0; i<rwidth; i++)
+	  pseudoternarysquarecolor
+	    ((PetscScalar) i/(rwidth-1), (PetscScalar) j/(rheight-1), scale,
+	     rgb+(j*rwidth + i) * bytes_per_pixel);
+      return 0;
+
     case FIELD_VECTOR:
       scale[0] = 0.;
       scale[1] = 1.;
@@ -395,6 +438,13 @@ int render_rgb_local_2d
 		 pixel);
 	      break;
 	    }
+	  case FIELD_TERNARY_SQUARE:
+	    {
+	      pseudoternarysquarecolor
+		(global_array [vecindex], global_array [vecindex+1], scale,
+		 pixel);
+	      break;
+	    }
 	  case FIELD_VECTOR:
 	  case FIELD_VECTOR+1:
 	    {
