@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.35 2004/11/30 14:35:54 hazelsct Exp $
+  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.36 2005/04/28 09:15:16 hazelsct Exp $
 
   This program views the output of a time series saved using
   +latex+{\tt IlluMultiSave()}.
@@ -46,10 +46,11 @@ double current_time;
 /* Window parameters and drawables */
 int width=0, height=0, bpp=3, nx, ny, dataview_count=1;
 GtkWidget *dataviews [1];
-guchar *rgbbuf [1] = { NULL }, scalar_rgb[120000], ternary_rgb[120000],
-  vector_rgb[120000], shear_rgb[120000];
-gboolean scalar_auto_set=TRUE, ternary_auto_set=TRUE, vector_auto_set=TRUE,
-  shear_auto_set=TRUE;
+guchar *rgbbuf [1] = { NULL }, scalar_rgb[120000], ternary_square_rgb[120000],
+  ternary_triangle_rgb[120000], *ternary_rgb, vector_rgb[120000],
+  shear_rgb[120000];
+gboolean scalar_auto_set=TRUE, ternary_auto_set=TRUE, ternary_square_set=FALSE,
+  vector_auto_set=TRUE, shear_auto_set=TRUE;
 double sizemag;
 PetscTruth transp;
 
@@ -57,7 +58,7 @@ PetscTruth transp;
 DA theda;
 Vec global;
 PetscScalar minmax[6] = { 0.,1., 0.,1., 0.,1. };
-PetscScalar scales[11]; /* 2 scalar, 2 vector, 6 ternary, 1 tensor */
+PetscScalar scales[15]; /* 2 scalar, 6 tritern, 4 sqtern, 2 vector, 1 tensor */
 field_plot_type *fieldtypes;
 int dimensions, num_fields, current_field, *field_index, num_variables[1],
   **variable_indices;
@@ -111,7 +112,7 @@ void render_dataviews ()
 	      if (scalar_auto_set)
 		{
 		  auto_scale (global_array, xm*ym, num_fields, current_field,
-			      fieldtypes [current_field], 2, scales);
+			      FIELD_SCALAR, 2, scales);
 		  snprintf (maxes[0], 19, "%g", scales[0]);
 		  snprintf (maxes[1], 19, "%g", scales[1]);
 		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
@@ -123,62 +124,95 @@ void render_dataviews ()
 		}
 	      ierr = render_rgb_local_2d
 		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, fieldtypes [current_field],
-		 scales, nx,ny, xs,ys, xm,ym);
-	      CHKERRQ (ierr);
+		 num_fields, current_field, FIELD_SCALAR,
+		 scales, nx,ny, xs,ys, xm,ym); CHKERRQ (ierr);
 	      break;
 
 	    case FIELD_TERNARY:
-	      if (ternary_auto_set)
+	    case FIELD_TERNARY_SQUARE:
+	      if (!ternary_square_set) /* Ternary triangle */
 		{
-		  auto_scale (global_array, xm*ym, num_fields, current_field,
-			      fieldtypes [current_field], 2, scales+2);
-		  snprintf (maxes[0], 19, "%g", scales[2]);
-		  snprintf (maxes[1], 19, "%g", scales[3]);
-		  snprintf (maxes[2], 19, "%g", scales[4]);
-		  snprintf (maxes[3], 19, "%g", scales[5]);
-		  snprintf (maxes[4], 19, "%g", scales[6]);
-		  snprintf (maxes[5], 19, "%g", scales[7]);
-		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						 (xml, "ternary_1A_entry")),
-				      maxes[0]);
-		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						 (xml, "ternary_1B_entry")),
-				      maxes[1]);
-		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						 (xml, "ternary_2A_entry")),
-				      maxes[2]);
-		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
-						 (xml, "ternary_2B_entry")),
-				      maxes[3]);
-		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						 (xml, "ternary_3A_entry")),
-				      maxes[4]);
-		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-						 (xml, "ternary_3B_entry")),
-				      maxes[5]);
+		  if (ternary_auto_set)
+		    {
+		      auto_scale (global_array, xm*ym, num_fields,
+				  current_field, FIELD_TERNARY, 2, scales+2);
+		      snprintf (maxes[0], 19, "%g", scales[2]);
+		      snprintf (maxes[1], 19, "%g", scales[3]);
+		      snprintf (maxes[2], 19, "%g", scales[4]);
+		      snprintf (maxes[3], 19, "%g", scales[5]);
+		      snprintf (maxes[4], 19, "%g", scales[6]);
+		      snprintf (maxes[5], 19, "%g", scales[7]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_1A_entry")),
+					  maxes[0]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_1B_entry")),
+					  maxes[1]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_2A_entry")),
+					  maxes[2]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
+						     (xml, "ternary_2B_entry")),
+					  maxes[3]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_3A_entry")),
+					  maxes[4]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_3B_entry")),
+					  maxes[5]);
+		    }
+		  ierr = render_rgb_local_2d
+		    (rgbbuf [viewnum], width,height,width, bpp, global_array,
+		     num_fields, current_field, FIELD_TERNARY, scales+2, nx,ny,
+		     xs,ys, xm,ym);
+		  CHKERRQ (ierr);
+		}
+	      else /* Ternary square */
+		{
+		  if (ternary_auto_set)
+		    {
+		      auto_scale (global_array, xm*ym, num_fields,
+				  current_field, FIELD_TERNARY_SQUARE, 2,
+				  scales+8);
+		      snprintf (maxes[0], 19, "%g", scales[8]);
+		      snprintf (maxes[1], 19, "%g", scales[9]);
+		      snprintf (maxes[2], 19, "%g", scales[10]);
+		      snprintf (maxes[3], 19, "%g", scales[11]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_1A_entry")),
+					  maxes[0]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_1B_entry")),
+					  maxes[1]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+						     (xml, "ternary_2A_entry")),
+					  maxes[2]);
+		      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
+						     (xml, "ternary_2B_entry")),
+					  maxes[3]);
+		    }
+		  ierr = render_rgb_local_2d
+		    (rgbbuf [viewnum], width,height,width, bpp, global_array,
+		     num_fields, current_field, FIELD_TERNARY_SQUARE, scales+8,
+		     nx,ny, xs,ys, xm,ym);
+		  CHKERRQ (ierr);
 		}
-	      ierr = render_rgb_local_2d
-		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, fieldtypes [current_field],
-		 scales+2, nx,ny, xs,ys, xm,ym);
-	      CHKERRQ (ierr);
 	      break;
 
 	    case FIELD_VECTOR:
 	      if (vector_auto_set)
 		{
 		  auto_scale (global_array, xm*ym, num_fields, current_field,
-			      fieldtypes [current_field], 2, scales+8);
-		  snprintf (maxes[0], 19, "%g", scales[9]);
+			      FIELD_VECTOR, 2, scales+12);
+		  snprintf (maxes[0], 19, "%g", scales[13]);
 		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
 						 (xml, "vector_max_entry")),
 				      maxes[0]);
 		}
 	      ierr = render_rgb_local_2d
 		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, fieldtypes [current_field],
-		 scales+8, nx,ny, xs,ys, xm,ym);
+		 num_fields, current_field, FIELD_VECTOR, scales+12, nx,ny,
+		 xs,ys, xm,ym);
 	      CHKERRQ (ierr);
 	      break;
 
@@ -186,16 +220,16 @@ void render_dataviews ()
 	      if (shear_auto_set)
 		{
 		  auto_scale (global_array, xm*ym, num_fields, current_field,
-			      fieldtypes [current_field], 2, scales+10);
-		  snprintf (maxes[0], 19, "%g", scales[10]);
+			      fieldtypes [current_field], 2, scales+14);
+		  snprintf (maxes[0], 19, "%g", scales[14]);
 		  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
 						 (xml, "shear_max_entry")),
 				      maxes[0]);
 		}
 	      ierr = render_rgb_local_2d
 		(rgbbuf [viewnum], width,height,width, bpp, global_array,
-		 num_fields, current_field, fieldtypes [current_field],
-		 scales+10, nx,ny, xs,ys, xm,ym);
+		 num_fields, current_field, FIELD_TENSOR_SHEAR, scales+14,
+		 nx,ny, xs,ys, xm,ym);
 	      CHKERRQ (ierr);
 	      break;
 	    }
@@ -407,6 +441,90 @@ void on_ternary_auto_checkbutton_toggled
     }
 }
 
+void on_ternary_square_checkbutton_toggled
+(GtkWidget *thebutton, gpointer user_data)
+{
+  int field;
+  char maxes [6][20];
+
+  ternary_square_set = gtk_toggle_button_get_active
+    (GTK_TOGGLE_BUTTON (thebutton));
+
+  for (field=0; field<num_fields; field++)
+    if (fieldtypes [field] == FIELD_TERNARY ||
+	fieldtypes [field] == FIELD_TERNARY_SQUARE)
+      fieldtypes [field] =
+	ternary_square_set ? FIELD_TERNARY_SQUARE : FIELD_TERNARY;
+  render_dataviews ();
+
+  if (ternary_square_set)
+    {
+      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3A_label"));
+      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3A_entry"));
+      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3B_label"));
+      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3B_entry"));
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_1A_label")), "Min A");
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_1B_label")), "Max B");
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_2A_label")), "Min A");
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_2B_label")), "Max B");
+      snprintf (maxes[0], 19, "%g", scales[8]);
+      snprintf (maxes[1], 19, "%g", scales[9]);
+      snprintf (maxes[2], 19, "%g", scales[10]);
+      snprintf (maxes[3], 19, "%g", scales[11]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_1A_entry")), maxes[0]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_1B_entry")), maxes[1]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_2A_entry")), maxes[2]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
+				     (xml, "ternary_2B_entry")), maxes[3]);
+
+      ternary_rgb = ternary_square_rgb;
+    }
+  else
+    {
+      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3A_label"));
+      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3A_entry"));
+      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_label"));
+      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_entry"));
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_1A_label")), "Magenta A");
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_1B_label")), "Magenta B");
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_2A_label")), "Cyan A");
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				     (xml, "ternary_2B_label")), "Cyan B");
+      snprintf (maxes[0], 19, "%g", scales[2]);
+      snprintf (maxes[1], 19, "%g", scales[3]);
+      snprintf (maxes[2], 19, "%g", scales[4]);
+      snprintf (maxes[3], 19, "%g", scales[5]);
+      snprintf (maxes[4], 19, "%g", scales[6]);
+      snprintf (maxes[5], 19, "%g", scales[7]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_1A_entry")), maxes[0]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_1B_entry")), maxes[1]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_2A_entry")), maxes[2]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
+				     (xml, "ternary_2B_entry")), maxes[3]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_3A_entry")), maxes[4]);
+      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				     (xml, "ternary_3B_entry")), maxes[5]);
+
+      ternary_rgb = ternary_triangle_rgb;
+    }
+  on_ternary_scale_area_expose_event
+    (glade_xml_get_widget (xml, "ternary_scale_area"), NULL, NULL);
+}
+
 void on_vector_auto_checkbutton_toggled
 (GtkWidget *thebutton, gpointer user_data)
 {
@@ -467,19 +585,19 @@ void on_scalar_max_entry_changed (GtkWidget *theentry, gpointer user_data)
 
 void on_ternary_1A_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", scales+2); }
+  sscanf (entrytext, "%lf", ternary_square_set ? scales+8 : scales+2); }
 
 void on_ternary_1B_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", scales+3); }
+  sscanf (entrytext, "%lf", ternary_square_set ? scales+9 : scales+3); }
 
 void on_ternary_2A_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", scales+4); }
+  sscanf (entrytext, "%lf", ternary_square_set ? scales+10 : scales+4); }
 
 void on_ternary_2B_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", scales+5); }
+  sscanf (entrytext, "%lf", ternary_square_set ? scales+11 : scales+5); }
 
 void on_ternary_3A_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
@@ -491,7 +609,7 @@ void on_ternary_3B_entry_changed (GtkWidget *theentry, gpointer user_data)
 
 void on_vector_max_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", scales+9); }
+  sscanf (entrytext, "%lf", scales+13); }
 
 void on_vector_symm_spinbutton_changed (GtkWidget *theentry,gpointer user_data)
 { int symmetry;
@@ -505,7 +623,7 @@ void on_vector_symm_spinbutton_changed (GtkWidget *theentry,gpointer user_data)
 
 void on_shear_max_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", scales+10); }
+  sscanf (entrytext, "%lf", scales+14); }
 
 /*++++++++++++++++++++++++++++++++++++++
   This loads the names of the files into a long list.
@@ -812,7 +930,9 @@ int main (int argc, char *argv[])
 
   /* Scale images */
   render_scale_2d (scalar_rgb, 200, 200, 3, FIELD_SCALAR, 1);
-  render_scale_2d (ternary_rgb, 200, 200, 3, FIELD_TERNARY, 1);
+  render_scale_2d (ternary_triangle_rgb, 200, 200, 3, FIELD_TERNARY, 1);
+  render_scale_2d (ternary_square_rgb, 200, 200, 3, FIELD_TERNARY_SQUARE, 1);
+  ternary_rgb = ternary_triangle_rgb;
   render_scale_2d (vector_rgb, 200, 200, 3, FIELD_VECTOR, 1);
   render_scale_2d (shear_rgb, 200, 200, 3, FIELD_TENSOR_SHEAR, 1);
 
